krb5 commit: Fix double-free in KDC TGS processing

ghudson at mit.edu ghudson at mit.edu
Mon Aug 7 17:54:37 EDT 2023


https://github.com/krb5/krb5/commit/88a1701b423c13991a8064feeb26952d3641d840
commit 88a1701b423c13991a8064feeb26952d3641d840
Author: Andreas Schneider <asn at samba.org>
Date:   Fri Aug 4 09:54:06 2023 +0200

    Fix double-free in KDC TGS processing
    
    When issuing a ticket for a TGS renew or validate request, copy only
    the server field from the outer part of the header ticket to the new
    ticket.  Copying the whole structure causes the enc_part pointer to be
    aliased to the header ticket until krb5_encrypt_tkt_part() is called,
    resulting in a double-free if handle_authdata() fails.
    
    [ghudson at mit.edu: changed the fix to avoid aliasing enc_part rather
    than check for aliasing before freeing; rewrote commit message]
    
    CVE-2023-39975:
    
    In MIT krb5 release 1.21, an authenticated attacker can cause a KDC to
    free the same pointer twice if it can induce a failure in
    authorization data handling.
    
    ticket: 9101 (new)
    tags: pullup
    target_version: 1.21-next

 src/kdc/do_tgs_req.c | 5 +++--
 1 file changed, 3 insertions(+), 2 deletions(-)

diff --git a/src/kdc/do_tgs_req.c b/src/kdc/do_tgs_req.c
index 6e4c8fa9f..0acc45850 100644
--- a/src/kdc/do_tgs_req.c
+++ b/src/kdc/do_tgs_req.c
@@ -1010,8 +1010,9 @@ tgs_issue_ticket(kdc_realm_t *realm, struct tgs_req_info *t,
     }
 
     if (t->req->kdc_options & (KDC_OPT_VALIDATE | KDC_OPT_RENEW)) {
-        /* Copy the whole header ticket except for authorization data. */
-        ticket_reply = *t->header_tkt;
+        /* Copy the header ticket server and all enc-part fields except for
+         * authorization data. */
+        ticket_reply.server = t->header_tkt->server;
         enc_tkt_reply = *t->header_tkt->enc_part2;
         enc_tkt_reply.authorization_data = NULL;
     } else {


More information about the cvs-krb5 mailing list